﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class Block : CodeNode {
    public override int MinimumArity { get { return 1; } }
    public override int MaximumArity { get { return 10; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    private Block(bool deserializing) : base(deserializing) { }
    private Block(Block original, Cloner cloner)
      : base(original, cloner) {
    }

    public Block()
      : base("Block", "A group of statements.") {
      this.Prefix = "{\r\n\t";
      this.Suffix = "\r\nexecute();\r\n}\r\n";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Block(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children) {
      string result = "";
      foreach (ISymbolicExpressionTreeNode c in children)
        result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
      return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
    }
  }
}